# $Id: parse.tcl,v 1.15 2000/03/07 14:37:07 gdmr Exp $
#
# Copyright 2000 Division of Informatics, University of Edinburgh
# 
# Permission to use, copy, modify, distribute, and sell this software and its
# documentation for any purpose is hereby granted without fee, provided that
# the above copyright notice appear in all copies and that both that
# copyright notice and this permission notice appear in supporting
# documentation, and that the name of the University of Edinburgh not be used
# in advertising or publicity pertaining to distribution of the software
# without specific, written prior permission.  The University of Edinburgh
# makes no representations about the suitability of this software for any
# purpose.  It is provided "as is" without express or implied warranty.
#
# Procedures used to parse the switch definition files.  These are structured
# in such a way that they're basically a collection of Tcl calls to things
# defined in this file.  There isn't a separate parser for them...

#
# Things it's sensible to have in a config-file
# Make this a proc so that calling it causes this
# file to be auto-loaded
#
proc parseCommands {} {
   list target target2 community unit port portskip vlan
}

#
# parse error reporting
#
set parseError 0

proc parseOK {} {
   global parseError
   if {$parseError} {
      return 0
   } else {
      return 1
   }
}

#
# General switch definitions
#
proc target {t} {
   global parseTarget
   if [info exists parseTarget] {
      puts stderr "Redefining target switch: $parseTarget -> $t"
   }
   set parseTarget $t
}

proc getTarget {} {
   global parseTarget
   set parseTarget
}

proc target2 {t} {
   global parseTarget2
   if [info exists parseTarget2] {
      puts stderr "Redefining target switch: $parseTarget2 -> $t"
   }
   set parseTarget2 $t
}

proc getTarget2 {} {
   global parseTarget2
   if [info exists parseTarget2] {
      set parseTarget2
   } else {
      return ""
   }
}

proc community {c} {
   global parseCommunity
   if [info exists parseCommunity] {
      puts stderr "Redefining community: $parseCommunity -> $c"
   }
   set parseCommunity $c
}

proc getCommunity {} {
   global parseCommunity
   set parseCommunity
}

#
# port definitions and things
#
proc textPort {p} {
   format "%d/%02d" [expr {$p / 100}] [expr {$p % 100}]
}

set parseUnit 1

proc unit {u} {
   global parseUnit
   set parseUnit $u
}

set parseSkipList {}

proc portskip {p} {
   global parseSkipList
   global parseUnit
   set pu [expr 100 * $parseUnit + $p]
   lappend parseSkipList $pu
}

proc getSkipList {} {
   global parseSkipList
   set parseSkipList
}

set parseLegalSpeeds {- 10HD 10FD 100HD 100FD}

proc port {p name {speed -} {vlan default} args} {
   global parseUnit
   global parseName
   global parseSpeed
   global parseLegalSpeeds
   global parseDefVlan
   global parseVlans
   global parseVlanTags
   global parseError
   set pu [expr 100 * $parseUnit + $p]
   if [info exists parseName($pu)] {
      puts stderr \
         "Redefining unit $parseUnit port $p: $name $speed $vlan $args"
      set parseError 1
      return
   }
   set parseName($pu) $name
   set us [string toupper $speed]
   if {[lsearch -exact $parseLegalSpeeds $us] >= 0} {
      set parseSpeed($pu) $us
   } else {
      puts stderr \
         "Speed \"$speed\" for unit $parseUnit port $p not in $parseLegalSpeeds"
      set parseError 1
   }
   if [info exists parseVlanTags($vlan)] {
      set parseDefVlan($pu) $vlan
   } else {
      puts stderr "Undefined VLAN \"$vlan\" for unit $parseUnit port $p"
      set parseError 1
   }
   foreach v $args {
      if {![info exists parseVlanTags($v)]} {
         puts stderr "Undefined VLAN \"$v\" for unit $parseUnit port $p"
         set parseError 1
      }
   }
   set parseVlans($pu) $args
}

proc getPortList {} {
   global parseName
   return [lsort -integer [array names parseName]]
}

proc portDefined {pu} {
   global parseName
   info exists parseName($pu)
}

proc getPortName {pu} {
   global parseName
   if [info exists parseName($pu)] {
      set parseName($pu)
   } else {
      return ""
   }
}

proc getPortSpeed {pu} {
   global parseSpeed
   # Beware: if the port hasn't been configured then
   # we won't have a speed for it.  In this case the
   # easiest thing is to assume that it's auto.
   if [info exists parseSpeed($pu)] {
      set parseSpeed($pu)
   } else {
      return "-"
   }
}

#
# VLAN definitions
#
proc getPortDefVlan {pu} {
   global parseDefVlan
   if [info exists parseDefVlan($pu)] {
      set parseDefVlan($pu)
   } else {
      return "default"
   }
}

proc getPortVlans {pu} {
   global parseVlans
   if [info exists parseVlans($pu)] {
      set parseVlans($pu)
   } else {
      return ""
   }
}

set parseVlanTags(default) 1
set parseVlanTags(1) default

proc vlan {name tag {fullName ""}} {
   global parseVlanTags
   global parseVlanFullNames
   global parseError
   if [info exists parseVlanTags($name)] {
      puts stderr "Redefining VLAN $name tag: $parseVlanTags($name) -> $tag"
      set parseError 1
   }
   set parseVlanTags($name) $tag
   if [info exists parseVlanTags($tag)] {
      puts stderr "Redefining VLAN $tag name: $parseVlanTags($tag) -> $name"
      set parseError 1
   }
   set parseVlanTags($tag) $name
   if {[string length $fullName] == 0} {
      set fullName $name
   }
   set parseVlanFullNames($name) $fullName
   set parseVlanFullNames($tag) $fullName
}

proc getVlanTagOrName {tn} {
   global parseVlanTags
   if [info exists parseVlanTags($tn)] {
      set parseVlanTags($tn)
   } else {
      return ""
   }
}

proc getVlanFullName {tn} {
   global parseVlanFullNames
   if [info exists parseVlanFullNames($tn)] {
      set parseVlanFullNames($tn)
   } else {
      return "*unknown vlan name*"
   }
}

proc getVlanTagList {} {
   global parseVlanTags
   lsort [array names parseVlanTags]
}

#
# Diagnostics
#
proc parseDump {} {
   global parseTarget
   global parseCommunity
   global parseName
   global parseVlans
   if [info exists parseTarget] {
      puts "Target: $parseTarget"
   } else {
      puts "Target: not set"
   }
   if [info exists parseCommunity] {
      puts "Community: $parseCommunity"
   } else {
      puts "Community: not set"
   }
   foreach pu [lsort -integer [array names parseName]] {
      puts "PU $pu: $parseName($pu)"
   }
   foreach pu [lsort -integer [array names parseVlans]] {
      puts "PU $pu: $parseVlans($pu)"
   }
   foreach x [array names parseVlanTags] {
      puts "Vlan $x: $parseVlanTags($x)"
   }
}

